import pickle
from pathlib import Path

from PIL import Image
from selenium import webdriver
from time import sleep


class Screenshotter(object):
    def __init__(
        self,
        base_url,
        cookies_file_path,
    ):
        self.cookies_file_path = cookies_file_path
        self.base_url = base_url
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--user-data-dir=chrome-data")
        self.driver = webdriver.Chrome(options=chrome_options)

        # if Path(self.cookies_file_path).exists():
        #     cookies = pickle.load(open(self.cookies_file_path, "rb"))

        #     self.driver.get(self.base_url)

        #     print(f"Loading cookies (current url is {self.driver.current_url})")

        #     from selenium.common import exceptions

        #     for cookie in cookies:
        #         try:
        #             self.driver.add_cookie(cookie)
        #             print("loaded cookie.")
        #         except exceptions.InvalidCookieDomainException as e:
        #             print(f"not loading cookie {str(e)}")

        #     print("Finished cookie load.")
        #     self.driver.refresh()
        # else:
        #     print("No cookies file present - no cookies loaded.")

    def save_cookies(self):
        return
        cookies = self.driver.get_cookies()
        pickle.dump(cookies, open(self.cookies_file_path, "wb"))

    def close_all(self):
        # close all open tabs
        if len(self.driver.window_handles) < 1:
            return
        for window_handle in self.driver.window_handles[:]:
            self.driver.switch_to.window(window_handle)
            self.driver.close()

    def do_shot(self, filename):
        d = Path("screenshots")
        d.mkdir(exist_ok=True)
        self.driver.save_screenshot(filename)

    def quit(self):
        self.save_cookies()
        # sleep(5)
        # self.close_all()
        self.driver.quit()


def crop_inwards(in_name, out_name, top, bottom, left, right):
    img = Image.open(in_name)
    width, height = img.size

    # crop
    # 10 pixels from the left
    # 20 pixels from the top
    # 30 pixels from the right
    # 40 pixels from the bottom

    cropped = img.crop(
        (
            left * 2,
            top* 2,
            width - (right * 2),
            height - (bottom * 2)
        )
    )
    cropped.save(out_name)


def crop_element(in_name, out_name, left, top, w, h):
    img = Image.open(in_name)
    width, height = img.size
    print(locals())
    # crop
    # 10 pixels from the left
    # 20 pixels from the top
    # 30 pixels from the right
    # 40 pixels from the bottom

    cropped = img.crop(
        (
            left*2,
            top*2,
            (left+w) * 2,
            (top+h) * 2
        )
    )
    cropped.save(out_name)


# if __name__ == '__main__':
#     selenium_object = SeleniumDriver()
#     driver = selenium_object.driver

#     if is_logged_in(driver):
#         print("Already logged in")
#     else:
#         print("Not logged in. Please log in.")

#     sleep(120)

#     selenium_object.quit()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['screenshotz']

package_data = \
{'': ['*']}

install_requires = \
['Pillow', 'pyyaml', 'selenium']

entry_points = \
{'console_scripts': ['screenshotz = screenshotz:command.do_command']}

setup_kwargs = {
    'name': 'screenshotz',
    'version': '3.8',
    'description': 'Basic screenshot scripting',
    'long_description': '# `screenshotz`: Easy screenshot scripting\n',
    'author': 'Robert Lechte',
    'author_email': 'robert.lechte@dpc.vic.gov.au',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)



import sys
from pathlib import Path
from time import sleep
from typing import List

import yaml

from .code import Screenshotter, crop_inwards, crop_element


def is_logged_in(driver, base_url):
    driver.get(base_url)
    for x in range(3):
        if base_url in driver.current_url:
            return True
        sleep(1.0)
    return False


def run(filename: str):
    p = Path(filename)

    if not p.exists():
        raise ValueError("config file does not exist")

    y = yaml.safe_load(p.read_text())
    # print(y)
    print(f"screenshotz: loading file from {p.resolve()}")

    base_url = y["base_url"]
    page_width = y.get("page_width", None)
    page_height = y.get("page_height", None)
    s = Screenshotter(base_url, "cookies.cookies")

    driver = s.driver

    if page_width or page_height:
        page_width = page_width or 800
        page_height = page_height or 800
        print(f'setting window size: ({page_width} x {page_height})')
        driver.set_window_size(page_width, page_height)

    if is_logged_in(driver, base_url):
        print("Already logged in")

        for page in y["pages"]:
            print("taking screenshot for page:")
            print(page)
            path = page.get("path", "/")
            driver.get(base_url + path)
            delay = page.get("delay", 1)

            sleep(delay)

            fname = page.get("out")

            fullscreen_name = "screenshots/" + fname + ".fullscreen.png"

            crops = [page.get(x, 0) for x in "top bottom left right".split()]

            xpath = page.get('xpath')

            scroll_down = page.get('scroll_down')

            y_adjust = page.get('y_adjust', 0)

            if scroll_down:
                pix = int(scroll_down)
                print(f'scrolling down {pix} pixels')
                driver.execute_script(f"window.scrollTo(0, {pix})")
                sleep(0.5)
                scroll_y = driver.execute_script('return window.scrollY;')
                print(f'Scroll position: {scroll_y}')

            s.do_shot(fullscreen_name)

            if xpath:
                e = driver.find_element_by_xpath(xpath)

                loc = e.location
                size = e.size

                x, y = loc['x'], loc['y']
                w, h = size['width'], size['height']

                y -= scroll_down

                if y_adjust:
                    print(f'Shifting screenshot y dimension by {y_adjust} px')
                    y += y_adjust
                crops = [x, y, w, h]
                print(f'Crop dimensions: {crops}')

                crop_element(fullscreen_name, "screenshots/" + fname, *crops)

            else:
                crop_inwards(fullscreen_name, "screenshots/" + fname, *crops)

    else:
        print("Not logged in. Please log in.")

        try:
            sleep(300)
        except KeyboardInterrupt:
            print("saving cookies and quitting...")
    s.quit()


def do_command() -> None:  # pragma: no cover
    try:
        args = parse_args(sys.argv[1:])
        # print(args)
        run(args)
    except ValueError:
        # print("Invalid args, please specify exactly two numbers")
        status = 1
        raise
    else:
        status = 0
    sys.exit(status)


def parse_args(args: List[str]) -> str:
    if args:
        return args[0]
    else:
        print("Error:You must spcify a config file.")
        raise ValueError
